@@ -11,6 +11,8 @@ use ExtUtils::Mkbootstrap;
 use File::Spec::Functions qw.catdir catfile.;
 use File::Path qw.mkpath.;
 
+my $pedantic = $ENV{AMBS_PEDANTIC} || 0;
+
 sub ACTION_pre_install {
     my $self = shift;
 
@@ -38,6 +40,7 @@ sub ACTION_pre_install {
 
     ## FIXME - usar o Module::Build para isto?
     for (qw.ujspell jspell-dict jspell-installdic.) {
+        $self->fix_shebang_line( catfile("scripts",$_) );
         $self->copy_if_modified( from   => catfile("scripts",$_),
                                  to_dir => catdir('blib','script'),
                                  flatten => 1 );
@@ -145,6 +148,10 @@ sub ACTION_create_objects {
     my $cbuilder = $self->cbuilder;
 
     my $c_files = $self->rscan_dir('src', qr/\.c$/);
+
+    my $extra_compiler_flags = $self->notes('ccurses');
+    $extra_compiler_flags = "-Wall -Werror $extra_compiler_flags" if $pedantic;
+
     for my $file (@$c_files) {
         my $object = $file;
         $object =~ s/\.c/.o/;
@@ -152,7 +159,7 @@ sub ACTION_create_objects {
         $cbuilder->compile(object_file  => $object,
                            source       => $file,
                            include_dirs => ["src"],
-                           extra_compiler_flags => $self->notes('ccurses'));
+                           extra_compiler_flags => $extra_compiler_flags);
     }
 }
 
